from .name_converter import NamingConventionType


class GeneratorNamingConventions:
    """
    Encapsulates the naming conventions which are used by the GeneratorBase class.
    """
    properties_naming_convention: NamingConventionType = None
    type_naming_convention: NamingConventionType = None
